/***************************************************************
 *
 * include/fs.h
 * Header for various filesystem libraries
 *
 * See LICENSE file included with this source for copyright
 * information.
 *
 ***************************************************************/

#ifndef ____FS_H_
#define ____FS_H_

#define AWF_RD 0x00000001
#define AWF_WR 0x00000002

typedef struct _AW_FILE {
	struct fs_def *def;
	struct fs_dir *d;
	uint32_t clus; // give cluster index
	uint32_t clus_count; // if 0xFFFFFFFF, this is a stream
	uint32_t clus_pos; // position in cluster
	uint32_t clus_len; // length of cluster
	uint32_t fposl, fposh; // position in file
	uint32_t flenl, flenh; // if both 0xFFFFFFFF, this is a stream
	uint32_t flags;
} AW_FILE;

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

// fs.c
struct fs_dir {
	char *fname;
	void *data;
};

struct fs_def {
	struct fs_dir *(*fs_getdir)(struct fs_dir *);
	struct fs_dir *(*fs_getfirstent)(struct fs_dir *);
	struct fs_dir *(*fs_getprevent)(struct fs_dir *);
	struct fs_dir *(*fs_getnextent)(struct fs_dir *);
};

#define FS_MAKEDRV_STD(name, fs_getdir, fs_getfirstent, fs_getprevent, fs_getnextent) \
	struct fs_dir *fs_getdir (struct fs_dir *); \
	struct fs_dir *fs_getfirstent (struct fs_dir *); \
	struct fs_dir *fs_getprevent (struct fs_dir *); \
	struct fs_dir *fs_getnextent (struct fs_dir *); \
	struct fs_def name = { \
		fs_getdir, \
		fs_getfirstent, \
		fs_getprevent, \
		fs_getnextent \
	}

struct fs_def_ll {
	uint32_t magic;
	struct fs_def *def;
	struct fs_def_ll *n;
};

// fs.c
extern void *fs_root_data;
extern struct fs_def *fs_root_def;

void FsAddFileSystem(uint32_t, struct fs_def *);
void FsAddFile(struct fs_dir *dir, char *fname, void *data);
void FsInit(void);

// ramfs.c
#define FS_MAGIC_RAMFS 0x52616D21 /* "Ram!" */
extern struct fs_def ramfs_def;

struct fs_ramfs_dir {
	struct fs_ramfs_cluster *f; // actual file data
	char *fname; // filename
	
	struct fs_ramfs_dir *u; // up (parent) (NULL => root dir)
	struct fs_ramfs_dir *d; // down (child / data)
	struct fs_ramfs_dir *b; // beginning
	struct fs_ramfs_dir *p; // previous
	struct fs_ramfs_dir *n; // next
};

struct fs_ramfs_cluster {
	uint32_t len, alen; // length of cluster, actual length of data
	struct fs_ramfs_cluster *b; // beginning
	struct fs_ramfs_cluster *p; // previous
	struct fs_ramfs_cluster *n; // next
	uint8_t data[];
};

void ramfs_init(void);

#endif
