/***************************************************************
 *
 * include/pci.h
 * Header for PCI subsystem
 *
 * See LICENSE file included with this source for copyright
 * information.
 *
 ***************************************************************/

#ifndef ____PCI_H_
#define ____PCI_H_

#include <stddef.h>

struct pcidev_t
{
	uint16_t vendor_id, device_id;
	uint16_t command, status;
	uint8_t revision;
	uint8_t cls_c, cls_b, cls_a;
	uint8_t cachesize, latency, headtype, bist;
	uint32_t bar[6];
	uint32_t cardbus_cis;
	uint16_t sub_vendor_id, sub_device_id;
	uint32_t exprom_address;
	uint32_t rsvd0[2];
	uint8_t irq_line, int_pin;
	uint8_t min_grant, max_latency;
	
	uint8_t extra_data[192];
} __attribute__((__packed__));

struct pcidev_ll
{
	struct pcidev_t dev;
	int port,b,s,f;
	struct pcidev_ll *p,*n;
};

#define PCI_CLASS_PRE20 0x00
#	define PCI_CLASS_PRE20_VGA 0x01

#define PCI_CLASS_DISP 0x03
#	define PCI_CLASS_DISP_V8 0x00
#		define PCI_CLASS_DISP_V8_VGA 0x00
#		define PCI_CLASS_DISP_V8_8514 0x00
#	define PCI_CLASS_DISP_XGA 0x01
#	define PCI_CLASS_DISP_OTHER 0x80

#define PCI_CLASS_SERIAL 0x0C
#	define PCI_CLASS_SERIAL_FIREWIRE 0x00
#	define PCI_CLASS_SERIAL_ACCESS 0x01
#	define PCI_CLASS_SERIAL_SSA 0x02
#	define PCI_CLASS_SERIAL_USB 0x03

#define PCI_CLASS_MISC 0xFF

// pciinit.c
extern struct pcidev_ll *pciroot;
extern BOOL pci_scanall;
void PciInit(void);
void PciScan(uint16_t);
uint32_t PciReadConf32(uint16_t, int, int, int, int);
void PciWriteConf32(uint16_t, int, int, int, int, uint32_t);
struct pcidev_ll *PciFindByClassA(struct pcidev_ll *, uint8_t);
struct pcidev_ll *PciFindByClassB(struct pcidev_ll *, uint8_t, uint8_t);
struct pcidev_ll *PciFindByClassC(struct pcidev_ll *, uint8_t, uint8_t, uint8_t);
struct pcidev_ll *PciFindByVendor(struct pcidev_ll *, uint16_t);
struct pcidev_ll *PciFindByProduct(struct pcidev_ll *, uint16_t, uint16_t);


#endif
