/***************************************************************
 *
 * include/usb.h
 * Header for USB subsystem
 *
 * See LICENSE file included with this source for copyright
 * information.
 *
 ***************************************************************/

#ifndef ____USB_H_
#define ____USB_H_

#include <stddef.h>
#include <stdint.h>

#include <pci.h>

/*
// Uhh, is this stuff really necessary?
// If it is, bug me and I'll uncomment / complete it.
//   -GM
struct usb_packet_token_t {
	uint8_t magic_sync;
	uint8_t packet_id;
	uint8_t addr; // 7 bits - address
	uint8_t endp; // 4 bits - endpoint
	uint8_t crc5; // 5 bits - CRC
	uint8_t magic_eop;
};

struct usb_packet_data_start_t {
	uint8_t magic_sync;
	uint8_t packet_id;
};
struct usb_packet_data_end_t {
	uint8_t magic_eop;
};
*/

// Nybble-wise: MSN = LSN ^ 0xF
// Or: MSN ^ LSN = 0xF
#define USB_PACKET_TOKEN_OUT   0xE1
#define USB_PACKET_TOKEN_IN    0x69
#define USB_PACKET_TOKEN_SOF   0xA5
#define USB_PACKET_TOKEN_SETUP 0x2D

#define USB_PACKET_DATA_DATA0  0xC3
#define USB_PACKET_DATA_DATA1  0x4B
#define USB_PACKET_DATA_DATA2  0x87
#define USB_PACKET_DATA_MDATA  0x0F

#define USB_PACKET_SHAKE_ACK   0xD2
#define USB_PACKET_SHAKE_NAK   0x5A
#define USB_PACKET_SHAKE_STALL 0x1E
#define USB_PACKET_SHAKE_NYET  0x96

#define USB_PACKET_SPEC_PRE    0x3C
#define USB_PACKET_SPEC_ERR    0x3C
#define USB_PACKET_SPEC_SPLIT  0x78
#define USB_PACKET_SPEC_PING   0xB4
// The PRE/ERR repetition is not a mistake.
// The USB 2.0 standard explicitly states this.

// Just check the lower 2 bits for these.
#define USB_PACKET_TOKEN 0x01
#define USB_PACKET_DATA  0x03
#define USB_PACKET_SHAKE 0x02
#define USB_PACKET_SPEC  0x00

// *** AWOS SPECIFICS ***
struct usbhost_ll
{
	uint16_t port, portmask;
	struct pcidev_ll *parent;
	
	// Frame list stuff.
	uint32_t *framelist; // & ~0xFFF
	uint32_t *framelist_malloc; // for free()ing
	uint16_t frameidx; // index to put stuff into
	uint32_t flags;
	
	struct usbhost_ll *n;
};

// Hmm, I wonder what these are for?
#define USB_AWOS_CONFIGURED	0x00000001

extern BOOL usb_irq_status_stfu; // Supress IRQ status messages?

// *** USB 2.0 SPECIFICS ***
// These are standard USB 2.0 names.
struct usb_packet_setup_t
{
	uint8_t bmRequestType;
	uint8_t bRequest;
	uint16_t wValue;
	uint16_t wIndex;
	uint16_t wLength;
};

// bmRequestType masks
#define USB_PACKET_RQTYPE_H2D 0x00
#define USB_PACKET_RQTYPE_D2H 0x80

#define USB_PACKET_RQTYPE_STANDARD 0x00
#define USB_PACKET_RQTYPE_CLASS    0x20
#define USB_PACKET_RQTYPE_VENDOR   0x40

#define USB_PACKET_RQTYPE_2DEVICE    0x00
#define USB_PACKET_RQTYPE_2INTERFACE 0x01
#define USB_PACKET_RQTYPE_2ENDPOINT  0x02
#define USB_PACKET_RQTYPE_2OTHER     0x03

// bRequest values
#define USB_PACKET_REQ_GET_STATUS	 0
#define USB_PACKET_REQ_CLEAR_FEATURE	 1
#define USB_PACKET_REQ_SET_FEATURE	 3
#define USB_PACKET_REQ_SET_ADDRESS	 5
#define USB_PACKET_REQ_GET_DESCRIPTOR	 6
#define USB_PACKET_REQ_SET_DESCRIPTOR	 7
#define USB_PACKET_REQ_GET_CONFIGURATION 8
#define USB_PACKET_REQ_SET_CONFIGURATION 9
#define USB_PACKET_REQ_GET_INTERFACE	 10
#define USB_PACKET_REQ_SET_INTERFACE	 11
#define USB_PACKET_REQ_SYNCH_FRAME	 12

// wIndex directions / endpoint stuff
#define USB_PACKET_EPIN  0x0080
#define USB_PACKET_EPOUT 0x0000
#define USB_PACKET_EPMSK 0x000F

// descriptor types
#define USB_DESC_DEVICE				1
#define USB_DESC_CONFIGURATION			2
#define USB_DESC_STRING				3
#define USB_DESC_INTERFACE			4
#define USB_DESC_ENDPOINT			5
#define USB_DESC_DEVICE_QUALIFIER		6
#define USB_DESC_OTHER_SPEED_CONFIGURATION	7
#define USB_DESC_INTERFACE_POWER		8

struct usb_packet_device_descriptor_t
{
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint16_t bcdUSB;
	uint8_t bDeviceClass;
	uint8_t bDeviceSubClass;
	uint8_t bDeviceProtocol;
	uint8_t bMaxPacketSize0; // 8, 16, 32, 64 are the only valid values
	uint16_t idVendor;
	uint16_t idProduct;
	uint16_t bcdDevice;
	uint8_t iManufacturer;
	uint8_t iProduct;
	uint8_t iSerialNumber;
	uint8_t bNumConfigurations;
	
};

struct usb_packet_device_qualifier_t
{
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint16_t bcdUSB;
	uint8_t bDeviceClass;
	uint8_t bDeviceSubClass;
	uint8_t bDeviceProtocol;
	uint8_t bMaxPacketSize0; // 8, 16, 32, 64 are the only valid values
	uint8_t bNumConfigurations;
	uint8_t _rsvd0;
};

struct usb_packet_config_descriptor_t
{
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint16_t wTotalLength;
	uint8_t bNumInterfaces;
	uint8_t bConfigurationValue;
	uint8_t iConfiguration;
	uint8_t bmAttributes;
	uint8_t bMaxPower; // 2mA units
};

struct usb_packet_other_speed_config_t
{
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint16_t wTotalLength;
	uint8_t bNumInterfaces;
	uint8_t bConfigurationValue;
	uint8_t iConfiguration;
	uint8_t bmAttributes;
	uint8_t bMaxPower; // 2mA units
};

// bmAttributes masks for config descriptors
#define USB_CONF_ATTR_RSV1A	0x80
#define USB_CONF_ATTR_SELFPWRD	0x40
#define USB_CONF_ATTR_RMTWAKE	0x20

struct usb_packet_interface_descriptor_t
{
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint8_t bInterfaceNumber;
	uint8_t bAlternateSetting;
	uint8_t bNumEndpoints;
	uint8_t bInterfaceClass;
	uint8_t bInterfaceSubClass;
	uint8_t bInterfaceProtocol;
	uint8_t iInterface;
};

struct usb_packet_endpoint_descriptor_t
{
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint8_t bEndpointAddress; // use USB_PACKET_EP* values
	uint8_t bmAttributes;
	uint16_t wMaxPacketSize; // 10:0
	uint8_t bInterval;
};

// bmAttributes masks for endpoint descriptors
#define USB_EP_ATTR_TTCONTROL	0x00
#define USB_EP_ATTR_TTISOCRON	0x01
#define USB_EP_ATTR_TTBULK	0x02
#define USB_EP_ATTR_TTINT	0x03
#define USB_EP_ATTR_ISNOSYNC	0x00
#define USB_EP_ATTR_ISASYNC	0x04
#define USB_EP_ATTR_ISADAPT	0x08
#define USB_EP_ATTR_ISSYNC	0x0C
#define USB_EP_ATTR_USDATAEP	0x00
#define USB_EP_ATTR_USFBKEP	0x10
#define USB_EP_ATTR_USIMPFBKEP	0x20
	// 0x30 is reserved.

#define USB_EP_MSKMAXPKTSZ	0x07FF
#define USB_EP_TRANPERMF1	0x0000
#define USB_EP_TRANPERMF2	0x0800
#define USB_EP_TRANPERMF3	0x1000
	// 0x1800 is reserved.


struct usb_packet_string_descriptor_langid_t
{
	uint8_t bLength; // len = (bLength-2)>>1
	uint8_t bDescriptorType;
	uint16_t wLANGID[];
};

struct usb_packet_string_descriptor_data_t
{
	uint8_t bLength; // strlen = bLength-2
	uint8_t bDescriptorType;
	char bString[];
};

// *** UHCI SPECIFICS ***
// Ports
#define USB_UHCI_USBCMD 	0x00 /* 16/8 R/W */
#define USB_UHCI_USBSTS 	0x02 /* 16/8 R/WC */
#define USB_UHCI_USBINTR	0x04 /* 16/8 R/W */
#define USB_UHCI_FRNUM		0x06 /* 16 R/W */
#define USB_UHCI_FLBASEADD	0x08 /* 32 R/W */
#define USB_UHCI_SOFMOD 	0x0C /* 8 R/W */
#define USB_UHCI_PORTSC1	0x10 /* 16 R/WC */
#define USB_UHCI_PORTSC2	0x12 /* 16 R/WC */

// Commands
#define USB_UHCI_CMD_MAXP	0x0080
#define USB_UHCI_CMD_CF 	0x0040
#define USB_UHCI_CMD_SWDBG	0x0020
#define USB_UHCI_CMD_FGR	0x0010
#define USB_UHCI_CMD_EGSM	0x0008
#define USB_UHCI_CMD_GRESET	0x0004
#define USB_UHCI_CMD_HCRESET	0x0002
#define USB_UHCI_CMD_RS 	0x0001

// Status
#define USB_UHCI_STS_HCHLT	0x0020
#define USB_UHCI_STS_HCPERR	0x0010
#define USB_UHCI_STS_HSERR	0x0008
#define USB_UHCI_STS_RSMDTC	0x0004
#define USB_UHCI_STS_USBERRINT	0x0002
#define USB_UHCI_STS_USBINT	0x0001

// USB interrupt enable flags
#define USB_UHCI_IE_SHORTPKT	0x0008
#define USB_UHCI_IE_ONCOMPLETE	0x0004
#define USB_UHCI_IE_RESUMEIE	0x0002
#define USB_UHCI_IE_CRCTO	0x0001

// Port status
#define USB_UHCI_PSTS_SUSPEND	0x1000
#define USB_UHCI_PSTS_PRTRST	0x0200
#define USB_UHCI_PSTS_LOWSPD	0x0100
#define USB_UHCI_PSTS_RSVD1A	0x0080 /* should read as 1 */
#define USB_UHCI_PSTS_RSMDTC	0x0040
#define USB_UHCI_PSTS_LNSN	0x0020
#define USB_UHCI_PSTS_LNSP	0x0010
#define USB_UHCI_PSTS_PRTEDCHG	0x0008
#define USB_UHCI_PSTS_PRTED	0x0004
#define USB_UHCI_PSTS_CONSTSCHG	0x0002
#define USB_UHCI_PSTS_CONSTS	0x0001

// Frame list pointer
#define USB_UHCI_FLP_QH 	0x00000002
#define USB_UHCI_FLP_TD 	0x00000000
#define USB_UHCI_FLP_TERM	0x00000001
// This is an AWOS internal for terminated tasks:
#define USB_UHCI_FLP_FREE	0x80000000


// Transfer descriptors
// 8 DWORDS, first 4 are set like so, last 4 are for software usage.
//	Link pointer
#define USB_UHCI_TDLP_QH	0x00000002
#define USB_UHCI_TDLP_TD	0x00000000
#define USB_UHCI_TDLP_TERM	0x00000001
#define USB_UHCI_TDLP_DFIRST	0x00000004
#define USB_UHCI_TDLP_BFIRST	0x00000000
//	Control / status
#define USB_UHCI_TDCS_SPD	0x20000000
#define USB_UHCI_TDCS_ERRLMT0	0x00000000
#define USB_UHCI_TDCS_ERRLMT1	0x08000000
#define USB_UHCI_TDCS_ERRLMT2	0x10000000
#define USB_UHCI_TDCS_ERRLMT3	0x18000000
#define USB_UHCI_TDCS_LOWSPD	0x04000000
#define USB_UHCI_TDCS_ISOCRON	0x02000000
#define USB_UHCI_TDCS_INTONCPL	0x01000000
#define USB_UHCI_TDCS_SACTIVE	0x00800000
#define USB_UHCI_TDCS_SSTALLED	0x00400000
#define USB_UHCI_TDCS_SDBUFERR	0x00200000
#define USB_UHCI_TDCS_SBABBLE	0x00100000
#define USB_UHCI_TDCS_SNAK	0x00080000
#define USB_UHCI_TDCS_SCRCTOERR	0x00040000
#define USB_UHCI_TDCS_SBS	0x00020000
#define USB_UHCI_TDCS_MSKALEN	0x000007FF
//	TD token
#define USB_UHCI_TDTK_MSKMAXLEN	0xFFE00000
#define USB_UHCI_TDTK_SHLMAXLEN	21
#define USB_UHCI_TDTK_DATATOG	0x00080000
#define USB_UHCI_TDTK_MSKENDPT	0x00078000
#define USB_UHCI_TDTK_SHLENDPT	15
#define USB_UHCI_TDTK_MSKDEVADD	0x00007F00
#define USB_UHCI_TDTK_SHLDEVADD	8
#define USB_UHCI_TDTK_MSKPKTID	0x000000FF
//	Next DWORD is the buffer pointer.
//	Last 4 DWORDs are for software usage.

// Queue header
//	First is link pointer, second is element pointer.
#define USB_UHCI_QH_QH   0x00000002
#define USB_UHCI_QH_TD   0x00000000
#define USB_UHCI_QH_TERM 0x00000001

// TD struct yay
struct usb_uhci_td_t
{
	uint32_t linkptr;
	uint32_t status;
	uint32_t token;
	uint32_t buffptr;
	
	// Extra DWORDs we can use.
	void *actualbuf; // Actual malloc() buffer.
	uint32_t extra[3];
};

// usb.c
extern struct usbhost_ll *usbroot, *usbrootend;
void UsbInit(void);
void UsbAddPciHost(struct pcidev_ll *);
void UsbUhciCmdSet(struct usbhost_ll *, uint16_t);
void UsbUhciCmdClear(struct usbhost_ll *, uint16_t);
uint16_t UsbUhciStatusRead(struct usbhost_ll *);
void UsbUhciStatusClear(struct usbhost_ll *, uint16_t);
void UsbUhciFrameAddrUpdate(struct usbhost_ll *);
void UsbUhciPortOut8(struct usbhost_ll *, uint16_t, uint8_t);
void UsbUhciPortOut16(struct usbhost_ll *, uint16_t, uint16_t);
void UsbUhciPortOut32(struct usbhost_ll *, uint16_t, uint32_t);
uint8_t UsbUhciPortIn8(struct usbhost_ll *, uint16_t);
uint16_t UsbUhciPortIn16(struct usbhost_ll *, uint16_t);
uint32_t UsbUhciPortIn32(struct usbhost_ll *, uint16_t);
struct usb_uhci_td_t *UsbUhciCreateTD(void *, uint16_t, uint8_t);
void UsbUhciDestroyTD(struct usb_uhci_td_t *);
BOOL UsbUhciScheduleTD(struct usbhost_ll *, struct usb_uhci_td_t *);
void irq_handler_usb(void *r);
void daemon_usbd(void);

#endif
