#################################################################
#
#kernel/bootup.asm
#Boot code for AWOS
#
#See LICENSE file included with this source for copyright
#information.
#
#################################################################

.intel_syntax noprefix

.extern k_main
.globl k_idle
.globl start
.globl __KiGetStack
.globl __KiGetCR0
.globl __KiGetCR2
.globl __KiGetCR3
.globl __KiGetTR

.text

	jmp start
start:
	mov esp, offset sys_stack
	jmp overmboot

.balign 4
mboot:
	.equ MULTIBOOT_PAGE_ALIGN, 1<<0
	.equ MULTIBOOT_MEMORY_INFO, 1<<1 
	.equ MULTIBOOT_AOUT_KLUDGE, 1<<16
	.equ MULTIBOOT_HEADER_MAGIC, 0x1BADB002
	.equ MULTIBOOT_HEADER_FLAGS, MULTIBOOT_PAGE_ALIGN | MULTIBOOT_MEMORY_INFO | MULTIBOOT_AOUT_KLUDGE
	.equ MULTIBOOT_CHECKSUM, -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
	.extern code, bss, end

	.long MULTIBOOT_HEADER_MAGIC
	.long MULTIBOOT_HEADER_FLAGS
	.long MULTIBOOT_CHECKSUM

overmboot:
	mov ebp,0
	push 0
	push ebx
	call k_main
k_idle:
	sti
	hlt
	jmp k_idle

__KiGetStack:
	mov eax,esp
	ret

__KiGetCR0:
	mov eax,cr0
	ret
__KiGetCR2:
	mov eax,cr2
	ret
__KiGetCR3:
	mov eax,cr3
	ret
__KiGetTR:
	str ax
	ret

.globl gdt_update
.extern gdt_ptr
gdt_update:
	lgdt [gdt_ptr]
	mov ax,0x10
	mov ds,ax
	mov es,ax
	mov fs,ax
	mov gs,ax
	mov ss,ax
	
	# XXX - work around as bug
	.att_syntax
	ljmp $0x8, $gdt_update_flush
	.intel_syntax noprefix

gdt_update_flush:
	ret

.globl tr_update
tr_update:
	mov ax,[esp+0x04]
	ltr ax
	ret

.globl __KiLoadSaneDescriptors
__KiLoadSaneDescriptors:
	mov ax,0x0010
	mov ds,ax
	mov es,ax
	mov fs,ax
	mov gs,ax
	ret

# We will exit real mode disgracefully and jump straight into the initial boot area.
# This is just in case the architecture does not support the keyboard way of rebooting.
.globl __KiRestartByJump
__KiRestartByJump:
	mov eax,cr0
	or eax,1
	xor eax,1
	mov cr0,eax
	
	# Flush the segments.
	xor ax,ax
	mov ds,ax
	mov es,ax
	mov fs,ax
	mov gs,ax
	mov ss,ax
	
	mov esp,0x0000FFFE
	
	# Now for the jump...
	jmp 0xFFFF, 0x0000

.globl v86t_start
.globl v86t_end
v86t_start:
.code16
	mov ax,0x0003
	int 0x10
	
	mov ah,0x0E
	mov bh,0x00
	mov si,offset v86t_message-v86t_start
	lp_printmsg:
		lodsb
		test al,al
		jz lpx_printmsg
		int 0x10
		jmp lp_printmsg
	lpx_printmsg:
	
	mov ax,500
	int 0x40
	
	iret
	
	v86t_message:
		# Sorry for the mess... >_> -- jacob
		.ascii "Hey AW, do you like this update?"
		.ascii " It's a bit insecure and slow but it'll do for now :/"
		.ascii " BTW, I thought this would be better than putting 'Hello World!', because that's just overused."
		.ascii " Anyways, this uses INT 0x10 for drawing to the screen, if you're curious."
		.ascii " Also, setting the video mode to graphics screws the font up when it returns to text,"
		.ascii " so I can't show you that, really."
		.byte 0x0D
		.byte 0x0A
		.ascii "   --GM"
		.byte 0x0D
		.byte 0x0A
		.byte 0x0D
		.byte 0x0A
		.ascii "P.S. The mode reset makes me realise how off my palette is >_>" 
		.byte 0x00
.code32
v86t_end:

.section .bss
	.fill 8192
sys_stack:
