/****************************************************************

cons/prim.c
Graphics primitives for people who are afraid of plot()

See LICENSE file included with this source for copyright
information.

****************************************************************/

// Bad jokes have been removed due to lack of ideas -GM

#include <kernel.h>
#include <cons.h>
#include <stddef.h>
#include <sys/types.h>

// Some internal functions...

// Swap two ints. Should really be in lib{k|c}, but this should do for now.
void prim_cswap(int *a, int *b)
{
	int t;
	
	t = *a; *a = *b; *b = t;
};

// Here's all the main stuff.

void GfxPlot(console_t *tty, int x, int y, int c)
{
	if(!(tty->flags & CONS_GFXMODE))
		return;
	tty->driver->gseek(tty, x, y);
	tty->driver->plot(tty, c);
};

void GfxRectFill(console_t *tty, int x1, int y1, int x2, int y2, int c)
{
	int x,xc,yc,xcc;
	void *p;
	
	if(!(tty->flags & CONS_GFXMODE))
		return;

	if(x1 < x2)
	{
		xcc = (++x2 - x1);
		x = x1;
	} else {
		xcc = (++x1 - x2);
		x = x2;
	};
	
	if(y1 < y2)
	{
		yc = (++y2 - y1);
		tty->driver->gseek(tty, x, y1);
	} else {
		yc = (++y1 - y2);
		tty->driver->gseek(tty, x, y2);
	};
	
	
	for(; yc!=0; yc--)
	{
		p = tty->mem;
		for(xc=xcc; xc!=0; xc--)
		{
			tty->driver->plot(tty, c);
			tty->driver->gincx(tty);
		};
		tty->mem = p;
		tty->driver->gincy(tty);
	};
};

void GfxRect(console_t *tty, int x1, int y1, int x2, int y2, int c)
{
	int xc,yc,i,x;
	
	if(!(tty->flags & CONS_GFXMODE))
		return;
	
	if(x1 < x2)
	{
		xc = x2 - x1;
		x = x1;
	} else {
		xc = x1 - x2;
		x = x2;
	};
	
	if(y1 < y2)
	{
		yc = y2 - y1;
		tty->driver->gseek(tty, x, y1);
	} else {
		yc = y1 - y2;
		tty->driver->gseek(tty, x, y2);
	};
	
	for(i=xc; i!=0; i--)
	{
		tty->driver->plot(tty, c);
		tty->driver->gincx(tty);
	};
	for(i=yc; i!=0; i--)
	{
		tty->driver->plot(tty, c);
		tty->driver->gincy(tty);
	};
	for(i=xc; i!=0; i--)
	{
		tty->driver->plot(tty, c);
		tty->driver->gdecx(tty);
	};
	for(i=yc; i!=0; i--)
	{
		tty->driver->plot(tty, c);
		tty->driver->gdecy(tty);
	};
};

void GfxHLine(console_t *tty, int x1, int y, int x2, int c)
{
	int xc;
	
	if(!(tty->flags & CONS_GFXMODE))
		return;
	
	if(x1 < x2)
	{
		xc = ++x2 - x1;
		tty->driver->gseek(tty, x1, y);
	} else {
		xc = ++x1 - x2;
		tty->driver->gseek(tty, x2, y);
	};
	
	for(; xc!=0; xc--)
	{
		tty->driver->plot(tty, c);
		tty->driver->gincx(tty);
	};
};

void GfxVLine(console_t *tty, int x, int y1, int y2, int c)
{
	int yc;
	
	if(!(tty->flags & CONS_GFXMODE))
		return;
	
	if(y1 < y2)
	{
		yc = ++y2 - y1;
		tty->driver->gseek(tty, x, y1);
	} else {
		yc = ++y1 - y2;
		tty->driver->gseek(tty, x, y2);
	};
	
	for(; yc!=0; yc--)
	{
		tty->driver->plot(tty, c);
		tty->driver->gincy(tty);
	};
};

// Here's one of the more interesting functions.
void GfxLine(console_t *tty, int x1, int y1, int x2, int y2, int c)
{
	int gx,gy,g;
	
	// How to implement the direction switch...
	void (*xmove)(console_t *tty) = tty->driver->gincx;
	
	if(!(tty->flags & CONS_GFXMODE))
		return;
	
	// If there is a simpler method, use it.
	if(x1 == x2) {
		GfxVLine(tty,x1,y1,y2,c);
	} else if(y1 == y2) {
		GfxHLine(tty,x1,y1,x2,c);
	} else {
		// x1,y1 must be on top.
		// x2,y2 must be on the bottom.
		if(y1 > y2)
		{
			prim_cswap(&x1,&x2);
			prim_cswap(&y1,&y2);
		}
		
		// Now, we must calculate the gradient components.
		gx = x2 - x1;
		gy = y2 - y1;
		
		// Which direction should we start plotting in?
		g = gy;
		if(x1 > x2)
		{
			xmove = tty->driver->gdecx;
			gx ^= -1; // Shortcut method of negating a number.
			g = 0;
		}
		
		// Seek to start.
		tty->driver->gseek(tty, x1, y1);
		
		// Now, plot along...
		while(y1 <= y2)
		{
			tty->driver->plot(tty, c);
			g -= gx;
			while(g < 0)
			{
				tty->driver->plot(tty, c);
				xmove(tty);
				g += gy;
			};
			tty->driver->gincy(tty);
			y1++;
		};
	};
};

// Introducting... fresh, new, Triangles!
// We need a few functions though...

// Assign 3 numbers by index. This is a lot more specific than the above.
// BE CAREFUL! This function is *NOT* 100% safe!
// *ONLY* USE 0,1,2 FOR i1,i2,i3!
void prim_assign3i(int i1, int i2, int i3, int *v1, int *v2, int *v3)
{
	int ibank[3] = {0,0,0};
	ibank[i1] = *v1;
	ibank[i2] = *v2;
	ibank[i3] = *v3;
	*v1 = ibank[0];
	*v2 = ibank[1];
	*v3 = ibank[2];
};

// Sort coordinates in order. Used in conjunction with prim_assign3i.
void prim_trisort(int *i1, int *i2, int *i3, int y1, int y2, int y3)
{
	// x1,y1 must be up the top.
	// x2,y2 must be in the middle.
	// x3,y3 must be on the bottom.
	
	while(y1 > y2 || y1 > y3 || y2 > y3)
	{
		if(y1 > y2)
		{
			prim_cswap(&y1,&y2);
			prim_cswap(i1,i2);
		};
		
		if(y2 > y3)
		{
			prim_cswap(&y3,&y2);
			prim_cswap(i3,i2);
		};
	};
};

// Righty ho, here's what we've got in store.

// GfxTriSolid / prim_tri_solid. This is the most basic you can get.
// There's only one colour involved here.
void prim_tri_solid(console_t *tty, int y, int ye, float xs, float xe, float gl, float gr, int *x1, int *x2, int c)
{
	int x;
	
	xs -= gl;
	xe -= gr;
	while((float)y <= ye)
	{
		xs += gl;
		xe += gr;
		x = (int)xs;
		tty->driver->gseek(tty, x, y);
		while((float)x <= xe)
		{
			tty->driver->plot(tty, c);
			tty->driver->gincx(tty);
			x++;
		};
		y++;
	};
	
	if(x1 != NULL)
		*x1 = (int)xs;
	
	if(x2 != NULL)
		*x2 = (int)xe;
};

void GfxTriSolid(console_t *tty, int x1, int y1, int x2, int y2, int x3, int y3, int c)
{
	int i1,i2,i3;
	float g12,g13,g23; // The gradient lines are dx/dy lines.
	
	if(!(tty->flags & CONS_GFXMODE))
		return;
	
	i1 = 0; i2 = 1; i3 = 2;
	prim_trisort(&i1,&i2,&i3,y1,y2,y3);
	prim_assign3i(i1,i2,i3,&x1,&x2,&x3);
	prim_assign3i(i1,i2,i3,&y1,&y2,&y3);
	
	// If we have to render a flat triangle, we will render an hline instead.
	// And rather crudely, too.
	if(y1 == y3)
	{
		GfxHLine(tty,x1,y1,x2,c);
		GfxHLine(tty,x2,y2,x3,c);
		GfxHLine(tty,x3,y3,x1,c);
		return;
	};
	
	g13 = (float)(x3 - x1)/(float)(y3 - y1);
	
	// Render the top half, if there is one.
	if(y1 != y2)
	{
		g12 = (float)(x2 - x1)/(float)(y2 - y1);
		if(g12 < g13)
			prim_tri_solid(tty, y1, y2, x1, x1, g12, g13, &x2, &x1, c);
		else
			prim_tri_solid(tty, y1, y2, x1, x1, g13, g12, &x1, &x2, c);
	};
	
	// Same for if there is a bottom half.
	if(y2 != y3)
	{
		g23 = (float)(x3 - x2)/(float)(y3 - y2);
		if(x2 < x1)
			prim_tri_solid(tty, y2, y3, x2, x1, g23, g13, NULL, NULL, c);
		else
			prim_tri_solid(tty, y2, y3, x1, x2, g13, g23, NULL, NULL, c);
	};
};

// If you want to shade, here's an algo if you can read it:
// d.Cabac / d.X = ( d.Xac/d.Y - d.Xab/d.Y ) / (Xb * ( Yc * d.Xac/d.Y - Yb * d.Xab/d.Y ))
