/***************************************************************
 *
 * cons/vesa.c
 * VESA VBE driver (32-bit part)
 *
 * See LICENSE file include with this source for copyright
 * information.
 * 
 ***************************************************************/

#include <kernel.h>
#include <stdlib.h>
#include <portio.h>
#include <cons.h>
#include <string.h>
#include <pci.h>

#include <v86.h>

uint32_t vesa_tpal[16] = {
	0x000000,
	0x0000AA,
	0x00AA00,
	0x00AAAA,
	0xAA0000,
	0xAA0055,
	0xAAAA00,
	0xAAAAAA,
	0x555555,
	0x0000FF,
	0x00FF00,
	0x00FFFF,
	0xFF0000,
	0xFF00FF,
	0xFFFF00,
	0xFFFFFF
};

/*define TTY_MAKEDRV_DUAL(nam, pop, push, tseek, gseek, gincx, gincy, gdecx, gdecy, tincx, tincy, tdecx, tdecy, vscroll, putch, plot, cls) \
	void pop (console_t *); \
	void push (console_t *); \
	void tseek (console_t *, int, int); \
	void gseek (console_t *, int, int); \
	void gincx (console_t *); \
	void gincy (console_t *); \
	void gdecx (console_t *); \
	void gdecy (console_t *); \
	void tincx (console_t *); \
	void tincy (console_t *); \
	void tdecx (console_t *); \
	void tdecy (console_t *); \
	void vscroll (console_t *); \
	void putch (console_t *, char); \
	void plot (console_t *, int); \
	void cls (console_t *); \
	struct cons_class nam = { \
		pop, push, tseek, gseek, gincx, gincy, gdecx, gdecy, tincx, tincy, tdecx, tdecy, vscroll, putch, plot, cls \
	}*/
TTY_MAKEDRV_DUAL(
	VgaVesa06,
	
	VgaVesa06_pop,
	VgaVesa06_push,
	VgaVesa06_tseek,
	VgaVesa06_gseek,
	VgaVesa06_gincx,
	VgaVesa06_gincy,
	VgaVesa06_gdecx,
	VgaVesa06_gdecy,
	VgaVesa06_tincx,
	VgaVesa06_tincy,
	VgaVesa06_tdecx,
	VgaVesa06_tdecy,
	VgaVesa06_vscroll,
	VgaVesa06_putch,
	VgaVesa06_plot,
	VgaVesa06_cls
);

struct vesa_info *vinfo;
uint32_t vesa_v86_getinfo,vesa_v86_getinfo_end;
uint32_t vesa_v86_getmodeinfo,vesa_v86_getmodeinfo_end;
uint32_t vesa_v86_setmode,vesa_v86_setmode_end;
uint32_t vesa_v86_gen,vesa_v86_gen_end;

console_t *VgaVesaModesetter(int w, int h, int bpp);

struct video_moder vmode_VgaVesa = {
	VgaVesaModesetter, NULL
};
struct vesa_mode_info *vesamode; // 65KB of padding.
int vesamode_count = 0;
/*                GRAPHICS                                TEXT

15-bit   7-bit    Resolution   Colors   15-bit   7-bit    Columns   Rows
mode     mode                           mode     mode
number   number                         number   number
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
100h     -        640x400      256      108h     -        80        60
101h     -        640x480      256
                                        109h     -        132       25
102h     6Ah      800x600      16       10Ah     -        132       43
103h     -        800x600      256      10Bh     -        132       50
                                        10Ch     -        132       60
104h     -        1024x768     16
105h     -        1024x768     256

106h     -        1280x1024    16
107h     -        1280x1024    256

10Dh     -        320x200      32K   (1:5:5:5)
10Eh     -        320x200      64K   (5:6:5)
10Fh     -        320x200      16.8M (8:8:8)
110h     -        640x480      32K   (1:5:5:5)
111h     -        640x480      64K   (5:6:5)
112h     -        640x480      16.8M (8:8:8)
113h     -        800x600      32K   (1:5:5:5)
114h     -        800x600      64K   (5:6:5)
115h     -        800x600      16.8M (8:8:8)
116h     -        1024x768     32K   (1:5:5:5)
117h     -        1024x768     64K   (5:6:5)
118h     -        1024x768     16.8M (8:8:8)
119h     -        1280x1024    32K   (1:5:5:5)
11Ah     -        1280x1024    64K   (5:6:5)
11Bh     -        1280x1024    16.8M (8:8:8)
11Ch     -        1600x1200    256
*/

void yield(void);
void VgaVesaInit(void)
{
	struct vesa_mode_info *m;
	struct vesa_info *v;
	struct pcidev_ll *pcidev;
	
	(void)pcidev;
	
	uint16_t *a16;
	int i;
	v = (void *)0x14000;
	
	vesamode = malloc(sizeof(struct vesa_mode_info) * 256);
	// We want VBE2 info when possible.
	v->vesa_sig[0] = 'V';
	v->vesa_sig[1] = 'B';
	v->vesa_sig[2] = 'E';
	v->vesa_sig[3] = '2';
	
	printf("vesa: Getting info...");
	memcpy((void *)0x8000, (void *)&vesa_v86_getinfo, (uint32_t)&vesa_v86_getinfo_end-(uint32_t)&vesa_v86_getinfo);
#ifndef __PPC
	KiV86Enter(0x0800, 0x0000, 0xFFFC);
#endif	/* !__PPC */
	vinfo = malloc(sizeof(struct vesa_info));
	memcpy(vinfo, v, sizeof(struct vesa_info));
	if((vinfo->ax & 0xFF) != 0x4F)
	{
		printf("failed - VESA not supported\n");
		return;
	};
	if((vinfo->ax & 0xFF00))
	{
		printf("failed - could not fetch VESA info block\n");
		return;
	};
	if(vinfo->vesa_sig[0] == 'V' && vinfo->vesa_sig[1] == 'E' && vinfo->vesa_sig[2] == 'S' && vinfo->vesa_sig[3] == 'A')
	{
		printf("done\n");
	} else {
		printf("failed - info signature invalid\n");
		return;
	};
	
	dprintf("- VESA VBE %X.%X detected\n", (vinfo->vesa_ver>>8) & 0xFF, vinfo->vesa_ver & 0xFF);
	dprintf("- OEM string (%04X:%04X): %s\n", vinfo->oemstr_seg, vinfo->oemstr_ptr, (void *)(vinfo->oemstr_ptr + (vinfo->oemstr_seg<<4)));
	dprintf("- Capabilities: %02X (", vinfo->capable);
	if(vinfo->capable & VESA_CAP_DAC8)
		dprintf(" dac8");
	if(vinfo->capable & VESA_CAP_VGA)
		dprintf(" vga");
	if(vinfo->capable & VESA_CAP_XRAMDAC)
		dprintf(" xramdac");
	if(vinfo->capable & VESA_CAP_STEREO)
		dprintf(" stereo");
	if(vinfo->capable & VESA_CAP_EXTSTEREO)
		dprintf(" extstereo");
	dprintf(" )\n");
	printf("- OEM thingies: %04X ", vinfo->oemrev);
	printf("| %04X:%04X ", vinfo->oemvend_seg, vinfo->oemvend_ptr);
	printf("| %04X:%04X ", vinfo->oemprod_seg, vinfo->oemprod_ptr);
	printf("| %04X:%04X\n", vinfo->oemprev_seg, vinfo->oemprev_ptr);
	
	dprintf("- Modes (%04X:%04X):", vinfo->mode_seg, vinfo->mode_ptr);
	a16 = (void *)(vinfo->mode_ptr + (vinfo->mode_seg<<4));
	
	/*pcidev = PciFindByClassA(NULL, PCI_CLASS_DISP);
	if(pcidev != NULL)
	{
		a16 = (void *)((uint32_t)a16 + (pcidev->dev).bar[0]);
	};*/
	
	dprintf("[%08X]",(uint32_t)a16);
	i = 64;
	
	while(*a16 != 0xFFFF && i)
	{
		dprintf(" %04X", *a16);
		a16++;
		i--;
	}
	
	memcpy((void *)0x8000, (void *)&vesa_v86_getmodeinfo, (uint32_t)&vesa_v86_getmodeinfo_end-(uint32_t)&vesa_v86_getmodeinfo);
	printf("vesa: obtaining mode info from 0x100 to 0x11C\n");
	m = (void *)0x14000;
	for(i=0x0100; i<=0x011C; i++)
	{
		m->cx = i;
#ifndef __PPC
		KiV86Enter(0x0800, 0x0000, 0xFFFC);
#endif /* !__PPC */
		if(m->ax == 0x004F)
		{
			//dprintf("Mode %04X: %ix%i (%i bpp)\n", i, m->w, m->h, m->bpp);
			vesamode[vesamode_count] = *m;
			vesamode_count++;
		}/* else {
			if((m->ax & 0xFF) != 0x4F)
				dprintf("Mode %04X failed - function (%04X)\n", i, m->ax);
			else
				dprintf("Mode %04X failed - error %02X\n", i, m->ax>>8);
		};*/
	};
	vga_modelist_end->n = &vmode_VgaVesa;
	vga_modelist_end = vga_modelist_end->n;
	//VgaClearScreen();
	// This USED to cause a crash. It doesn't now. YES! :D :D :D
	dprintf("Mode list:\n");
	for(i=0; i<vesamode_count; i++)
	{
		dprintf("| %04X: %ix%i (%i bpp)", vesamode[i].cx, vesamode[i].w, vesamode[i].h, vesamode[i].bpp);
	};
	printf("\n");
};

void VgaVesaSetByList(console_t *tty, int idx)
{
	struct vesa_crtc_info *v;
	
	(void)tty;
	v = (void *)0x14000;
	v->bx = vesamode[idx].cx;
	memcpy((void *)0x8000, (void *)&vesa_v86_setmode, (uint32_t)&vesa_v86_setmode_end-(uint32_t)&vesa_v86_setmode);
#ifndef __PPC
	KiV86Enter(0x0800, 0x0000, 0xFFFC);
#endif /* !__PPC */
	dprintf("AX: %04X\n", v->ax);
};

void csleep(uint32_t);
void VgaVesaTextMode(console_t *tty)
{
	struct vesa_crtc_info *v;
	
	(void)tty;
	v = (void *)0x14000;
	v->bx = 0x0003;
	memcpy((void *)0x8000, (void *)&vesa_v86_setmode, (uint32_t)&vesa_v86_setmode_end-(uint32_t)&vesa_v86_setmode);
#ifndef __PPC
	KiV86Enter(0x0800, 0x0000, 0xFFFC);
#endif /* !__PPC */
	csleep(100);
};


void VgaVesa06_scr_fix_byval(console_t *tty, uint32_t mem)
{
	struct vesa_genreg *v;
	uint32_t dx;
	
	tty->i1 = mem & 0xFFFF0000;
	dx = tty->i1/tty->i2;
	//dprintf("[mem=%08X base=%08X page=%08X gran=%08X]", mem, tty->i1, dx, tty->i2);
	v = (void *)0x14000;
	memcpy((void *)0x8000, (void *)&vesa_v86_gen, (uint32_t)&vesa_v86_gen_end-(uint32_t)&vesa_v86_gen);
#ifndef __PPC
	if((tty->i4 & 1) != 0)
	{
		v->ax = 0x4F05;
		v->bx = 0x0000;
		v->dx = (uint16_t)dx;
		KiV86Enter(0x0800, 0x0000, 0xFFFC);
	};
	if((tty->i4 & 2) != 0)
	{
		v->ax = 0x4F05;
		v->bx = 0x0001;
		v->dx = (uint16_t)dx;
		KiV86Enter(0x0800, 0x0000, 0xFFFC);
	};
#endif /* !__PPC */
}
void VgaVesa06_scr_fix(console_t *tty)
{
	if(!(tty->flags & CONS_ACTIVE) || (tty->i3 & VESA_MODEATTR_LINEAR))
		return;
	
	if(	   (uint32_t)tty->mem >= (uint32_t)(tty->i1 + tty->ws)
		|| (uint32_t)tty->mem < (uint32_t)tty->i1)
	{
		VgaVesa06_scr_fix_byval(tty, (uint32_t)tty->mem);
	};
};

void VgaVesa06_pop(console_t *tty)
{
	int i;
	
	if(!(tty->flags & CONS_ACTIVE))
	{
		tty->currmem = tty->vrammem;
		tty->flags |= CONS_ACTIVE;
		
		VgaVesaSetByList(tty, tty->mode);
		
		if(tty->flags & CONS_PERSIST)
		{
			for(i=0; i<tty->memlen; i += tty->ws)
			{
				VgaVesa06_scr_fix_byval(tty, i);
				memcpy((void *)tty->ttymem, (void *)tty->vrammem, tty->ws);
			};
		};
		VgaVesa06_scr_fix_byval(tty, 0);
	};
};

void VgaVesa06_push(console_t *tty)
{
	int i;
	
	if(tty->flags & CONS_ACTIVE)
	{
		if(tty->flags & CONS_PERSIST)
		{
			for(i=0; i<tty->memlen; i += tty->ws)
			{
				VgaVesa06_scr_fix_byval(tty, i);
				memcpy((void *)tty->vrammem, (void *)tty->ttymem, tty->ws);
			};
		};
		tty->currmem = tty->ttymem;
		tty->flags &= ~CONS_ACTIVE;
		if(tty->driver != cons_currclass)
		{
			VgaVesaTextMode(tty);
		};
		VgaVesa06_scr_fix_byval(tty, 0);
	};
};

void VgaVesa06_tseek(console_t *tty, int x, int y)
{
	tty->tx = x;
	tty->ty = y;
	tty->gx = x*tty->fw;
	tty->gy = y*tty->fh;
	tty->mem = (void *)(y*tty->tline + x*tty->pitch*tty->fw);
	VgaVesa06_scr_fix(tty);
};

void VgaVesa06_gseek(console_t *tty, int x, int y)
{
	tty->gx = x;
	tty->gy = y;
	tty->mem = (void *)(y*tty->gline + x*tty->pitch);
	VgaVesa06_scr_fix(tty);
};

void VgaVesa06_gincx(console_t *tty)
{
	tty->gx++;
	//VgaVesa06_gseek(tty, tty->gx, tty->gy);
	tty->mem += tty->pitch;
	VgaVesa06_scr_fix(tty);
};
void VgaVesa06_gdecx(console_t *tty)
{
	tty->gx--;
	//VgaVesa06_gseek(tty, tty->gx, tty->gy);
	tty->mem -= tty->pitch;
	VgaVesa06_scr_fix(tty);
};

void VgaVesa06_gincy(console_t *tty)
{
	tty->gy++;
	//VgaVesa06_gseek(tty, tty->gx, tty->gy);
	tty->mem += tty->gline;
	VgaVesa06_scr_fix(tty);
};
void VgaVesa06_gdecy(console_t *tty)
{
	tty->gy--;
	//VgaVesa06_gseek(tty, tty->gx, tty->gy);
	tty->mem -= tty->gline;
	VgaVesa06_scr_fix(tty);
};

void VgaVesa06_tincx(console_t *tty)
{
	tty->tx++;
	//VgaVesa06_tseek(tty, tty->tx, tty->ty);
	tty->gx += tty->fw;
	tty->mem += tty->pitch * tty->fw;
	VgaVesa06_scr_fix(tty);
};
void VgaVesa06_tdecx(console_t *tty)
{
	tty->tx--;
	//VgaVesa06_tseek(tty, tty->tx, tty->ty);
	tty->gx -= tty->fw;
	tty->mem -= tty->pitch * tty->fw;
	VgaVesa06_scr_fix(tty);
};

void VgaVesa06_tincy(console_t *tty)
{
	tty->ty++;
	//VgaVesa06_tseek(tty, tty->tx, tty->ty);
	tty->gy += tty->fh;
	tty->mem += tty->tline;
	VgaVesa06_scr_fix(tty);
};
void VgaVesa06_tdecy(console_t *tty)
{
	tty->ty--;
	//VgaVesa06_tseek(tty, tty->tx, tty->ty);
	tty->gy -= tty->tline;
	tty->mem -= tty->tline;
	VgaVesa06_scr_fix(tty);
};

void VgaVesa06_vscroll(console_t *tty) // This is lazy, however, I need this thing to work for now. -GM
{
	VgaVesa06_tseek(tty, tty->tx, 0);
};

void VgaVesa06_plot_intern(console_t *tty, int c)
{
	switch(tty->bpp)
	{
		case 24:
			if(tty->i3 & VESA_MODEATTR_LINEAR)
			{
				*(uint16_t *)((uint32_t)tty->mem + (uint32_t)tty->currmem - tty->i1) = c & 0xFFFF;
				tty->mem += 2;
				*(uint8_t *)((uint32_t)tty->mem + (uint32_t)tty->currmem - tty->i1) = (c>>16) & 0xFF;
				tty->mem -= 2;
			}
			else
			{
				VgaVesa06_scr_fix(tty);
				*(uint8_t *)((uint32_t)tty->mem + (uint32_t)tty->currmem - tty->i1) = c & 0xFF;
				tty->mem++;
				VgaVesa06_scr_fix(tty);
				*(uint8_t *)((uint32_t)tty->mem + (uint32_t)tty->currmem - tty->i1) = (c>>8) & 0xFF;
				tty->mem++;
				VgaVesa06_scr_fix(tty);
				*(uint8_t *)((uint32_t)tty->mem + (uint32_t)tty->currmem - tty->i1) = (c>>16) & 0xFF;
				tty->mem--;
				tty->mem--;
				VgaVesa06_scr_fix(tty);
			};
			break;
		case 32:
			VgaVesa06_scr_fix(tty);
			// This is the colour model on my nVidia GeForce 5200 AGP. -GM
			*(uint32_t *)((uint32_t)tty->mem + (uint32_t)tty->currmem - tty->i1) = c;
			break;
	};
};

void VgaVesa06_putch(console_t *tty, char v) // Like VgaGfx, this only supports fonts of width 8. -GM
{
	char b;
	void *m;
	int x,y,i;
	i = v * tty->fh;
	m = tty->mem;
	tty->mem += tty->tline;
	VgaVesa06_scr_fix(tty);
	tty->mem = m;
	VgaVesa06_scr_fix(tty);
	for(y=0; y<tty->fh; y++)
	{
		b = g_8x8_font[i];
		for(x=0; x<8; x++)
		{
			if(b & 0x80)
				VgaVesa06_plot_intern(tty, vesa_tpal[tty->color & 0xF]);
			else if(!(tty->flags & CONS_TRANS))
				VgaVesa06_plot_intern(tty, vesa_tpal[(tty->color>>4) & 0xF]);
			VgaVesa06_gincx(tty);
			b <<= 1;
		};
		VgaVesa06_gseek(tty, tty->gx-8, tty->gy);
		VgaVesa06_gincy(tty);
		i++;
	};
	VgaVesa06_gseek(tty, tty->gx, tty->gy - tty->fh);
};

void VgaVesa06_plot(console_t *tty, int c)
{
	VgaVesa06_plot_intern(tty, c);
};

void VgaVesa06_cls(console_t *tty)
{
	int i;
	
	for(i=0; i<tty->memlen; i += tty->ws)
	{
		VgaVesa06_scr_fix_byval(tty, i);
		memset((void *)tty->vrammem, 0, tty->ws);
	};
	VgaVesa06_scr_fix_byval(tty, 0);
	tty->gx = tty->gy = tty->tx = tty->ty = 0;
};

console_t *VgaVesaModesetter(int w, int h, int bpp)
{
	console_t *tty;
	int i,m;
	m = -1;
	for(i=0;i<vesamode_count;i++)
	{
		if(vesamode[i].w == w && vesamode[i].h == h && vesamode[i].bpp == bpp)
		{
			dprintf("vesa: Found mode %04X at index %i\n", vesamode[i].cx, i);
			m = i;
		};
	}
	if(m == -1)
		return NULL;
	
	// VgaVesa06 gets its name from the colour model.
	if((vesamode[m].bpp == 24 || vesamode[m].bpp == 32) && vesamode[m].mem_model == 0x06)
	{
		dprintf("vesa: Using VgaVesa06 driver\n");
		dprintf("- Flags: (");
		if(vesamode[m].mode_attr & VESA_MODEATTR_SUPPORTED)
			dprintf(" supp");
		if(vesamode[m].mode_attr & VESA_MODEATTR_BIOSOUTPUT)
			dprintf(" biosout");
		if(vesamode[m].mode_attr & VESA_MODEATTR_COLOR)
			dprintf(" colour");
		if(vesamode[m].mode_attr & VESA_MODEATTR_GRAPHICS)
			dprintf(" gfx");
		if(vesamode[m].mode_attr & VESA_MODEATTR_NONVGA)
			dprintf(" nonvga");
		if(vesamode[m].mode_attr & VESA_MODEATTR_NONBANK)
			dprintf(" nonbank");
		if(vesamode[m].mode_attr & VESA_MODEATTR_LINEAR)
			dprintf(" linear");
		if(vesamode[m].mode_attr & VESA_MODEATTR_DOUBLESCAN)
			dprintf(" 2scan");
		if(vesamode[m].mode_attr & VESA_MODEATTR_INTERLACE)
			dprintf(" ilace");
		if(vesamode[m].mode_attr & VESA_MODEATTR_HW3BUF)
			dprintf(" hw3b");
		if(vesamode[m].mode_attr & VESA_MODEATTR_HWSTEREO)
			dprintf(" hwstereo");
		if(vesamode[m].mode_attr & VESA_MODEATTR_DUALDISP)
			dprintf(" hwdual");
		dprintf(" )\n");
		dprintf("- Window A: (");
		if(vesamode[m].wa_attr & VESA_WATTR_SUPPORTED)
			dprintf(" supp");
		if(vesamode[m].wa_attr & VESA_WATTR_READ)
			dprintf(" read");
		if(vesamode[m].wa_attr & VESA_WATTR_WRITE)
			dprintf(" write");
		dprintf(" ) seg=%04X\n", vesamode[m].wa_seg);
		dprintf("- Window B: (");
		if(vesamode[m].wb_attr & VESA_WATTR_SUPPORTED)
			dprintf(" supp");
		if(vesamode[m].wb_attr & VESA_WATTR_READ)
			dprintf(" read");
		if(vesamode[m].wb_attr & VESA_WATTR_WRITE)
			dprintf(" write");
		dprintf(" ) seg=%04X\n", vesamode[m].wb_seg);
		dprintf("- Windows: gran=%iKB size=%iKB\n", vesamode[m].w_gran, vesamode[m].w_size);
		dprintf("- Linear pointer: %08X\n", vesamode[m].phys_base_ptr);
		dprintf("- x:R:G:B: %i:%i:%i:%i\n", vesamode[m].rsv_size, vesamode[m].r_size, vesamode[m].g_size, vesamode[m].b_size);
		
		tty = malloc(sizeof(console_t));
		tty->mem = (void *)0; // In this driver, tty->mem is an offset from VRAM.
		tty->i3 = vesamode[m].mode_attr;
		tty->ws = 0;
		
		// Be careful with the barlfb flag!!!
		// It *CAN* work, but you REALLY should wait 'till you're desperate.
		if (k_kcmdline_check("barlfb") != NULL)
		{
			dprintf("vesa: Attempting to get linear frame buffer from PCI.\n");
			struct pcidev_ll *pcidev;
			
			pcidev = PciFindByClassA(NULL, PCI_CLASS_DISP);
			if(pcidev != NULL)
			{
				int h,l;
				dprintf("vesa: Found PCI device\n");
				i = (pcidev->dev).bar[0];
				dprintf("BAR0[I]: %08X\n", i);
				
				cli();
				
				PciWriteConf32(pcidev->port, pcidev->b, pcidev->s, pcidev->f, 0x10, 0xFFFFFFF0 | (i & 0xF));
				h = PciReadConf32(pcidev->port, pcidev->b, pcidev->s, pcidev->f, 0x10);
				dprintf("BAR0[H]: %08X\n", h);
				PciWriteConf32(pcidev->port, pcidev->b, pcidev->s, pcidev->f, 0x10, 0x00000000 | (i & 0xF));
				l = PciReadConf32(pcidev->port, pcidev->b, pcidev->s, pcidev->f, 0x10);
				dprintf("BAR0[L]: %08X\n", l);
				PciWriteConf32(pcidev->port, pcidev->b, pcidev->s, pcidev->f, 0x10, i);
				i = (pcidev->dev).bar[0] = PciReadConf32(pcidev->port, pcidev->b, pcidev->s, pcidev->f, 0x10);
				
				sti();
				
				dprintf("BAR0[R]: %08X\n", i);
				
				tty->vrammem = (void *)(i & ~0xF);
				l = -((h^l) & ~0xF);
				tty->i2 = tty->ws = l;
				
				dprintf("Size: %08X - %iKB, %iMB\n", l, l>>10, l>>20);
			};
		};
		
		if(tty->ws == 0)
		{
			if((vesamode[m].mode_attr & VESA_MODEATTR_LINEAR) && k_kcmdline_check("nolfb") == NULL) {
				dprintf("vesa: Attempting to use linear frame buffer.\n");
				tty->vrammem = (void *)(vesamode[m].phys_base_ptr);
				// Just in case, but kinda stupid.
				// Was an attempt to "fix" a bug. -GM
				//if(k_kcmdline_check("lfbalign") != NULL)
				//	tty->vrammem = (void *)((uint32_t)(tty->vrammem) & ~0x3FF); // Assume 1KB alignment
				
				tty->i2 = tty->ws = vesamode[m].bytes_scanline * h;
			} else {
				// We must not use the VESA_MODEATTR_LINEAR flag if we use banked modes,
				// else bad things happen.
				tty->i3 &= ~VESA_MODEATTR_LINEAR;
				tty->vrammem = (void *)(vesamode[m].wa_seg << 4);
				tty->ws = vesamode[m].w_size<<10;
				tty->i2 = vesamode[m].w_gran<<10;
			};
		} else {
			// If we are using barlfb, set the VESA_MODEATTR_LINEAR flag.
			tty->i3 |= VESA_MODEATTR_LINEAR;
			dprintf("vesa: All clear for PCI LFB.\n");
		};
		tty->i1 = 0;
		tty->currmem = tty->ttymem = tty->vrammem;
		tty->wc = 1;
		tty->tx = tty->ty = tty->gx = tty->gy = 0;
		tty->fw = 8; tty->fh = 8;
		tty->gw = w; tty->gh = h;
		tty->tw = w>>3; tty->th = h>>3;
		tty->color = 0x0F; tty->bcolor = 0x0;
		tty->flags = CONS_TEXTMODE | CONS_GFXMODE | CONS_TRANS;
		tty->tablen = 8;
		tty->font = &g_8x8_font[0];
		tty->bpp = bpp;
		tty->pitch = (bpp+7)>>3;
		tty->modetype = 0x06;
		tty->driver = &VgaVesa06;
		tty->mode = m;
		tty->gline = vesamode[m].bytes_scanline;
		tty->tline = vesamode[m].bytes_scanline * tty->fh;
		tty->memlen = tty->pitch * tty->gline * h;
		
		
		tty->i4 = 0;
		if(vesamode[m].wa_attr & VESA_WATTR_SUPPORTED)
			tty->i4 |= 1;
		
		if(vesamode[m].wb_attr & VESA_WATTR_SUPPORTED)
			tty->i4 |= 2;
		
		return tty;
	}
	return NULL;
};
