/****************************************************************

kernel/include/cpuinfo
CPU info declarations for AWOS

See the LICENSE file included with this source for copyright
information.

****************************************************************/

#ifndef __CPU_INFO_H
#define __CPU_INFO_H

/* These defines go to KiCpuInfo.features.c_flags */
/** supports SSE3 **/
#define CPUCAP_C_SSE3		0x00000001
/** support MONITOR/MWAIT **/
#define CPUCAP_C_MONITOR	0x00000008
/** qualified debug store (???) **/
#define CPUCAP_C_DSCPL		0x00000010
/** enhanced SpeedStep technology **/
#define CPUCAP_C_EST		0x00000080
/** thermal monitor 2 (???) **/
#define CPUCAP_C_TM2		0x00000010
/** L1 cache context can be set **/
#define CPUCAP_C_CNXTID		0x00000040

/* These defines go to KiCpuInfo.features.d_flags */
/** FPU on chip **/
#define CPUCAP_D_FPU		0x00000001
/** V86 mode enhancement support **/
#define CPUCAP_D_VME		0x00000002
/** debugging extension support **/
#define CPUCAP_D_DE		0x00000004
/** page size extension (e.g. 4MiB pages supported) **/
#define CPUCAP_D_PSE		0x00000008
/** time stamp counter support **/
#define CPUCAP_D_TSC		0x00000010
/** MSR support **/
#define CPUCAP_D_MSR		0x00000020
/** >32-bit address support **/
#define CPUCAP_D_PAE		0x00000040
/** machine check support **/
#define CPUCAP_D_MCE		0x00000080
/** cmpxchg8b support **/
#define CPUCAP_D_CX8		0x00000100
/** APIC on chip **/
#define CPUCAP_D_APIC		0x00000200
//bit 10 is reserved
/** SYS[ENTER/EXIT] supported **/
#define CPUCAP_D_SEP		0x00000800
/** MTRR registers are supported **/
#define CPUCAP_D_MTRR		0x00001000
/** global page table support **/
#define CPUCAP_D_PGE		0x00002000
/** machine check common error reporting support **/
#define CPUCAP_D_MCA		0x00004000
/** conditional move support **/
#define CPUCAP_D_CMOV		0x00008000
/** page attribute table supported **/
#define CPUCAP_D_PAT		0x00010000
/** 36-bit page sizes **/
#define CPUCAP_D_PSE36		0x00020000
/** processor serial number **/
#define CPUCAP_D_PSN		0x00040000
/** CLFLUSH supported **/
#define CPUCAP_D_CLFSH		0x00080000
//bit 20 is reserved
/** debug store support (???) **/
#define CPUCAP_D_DS		0x00200000
/** thermal MSRs **/
#define CPUCAP_D_ACPI		0x00400000
/** MMX technology support **/
#define CPUCAP_D_MMX		0x00800000
/** floating point context saves (FX[SAVE/RSTOR]) **/
#define CPUCAP_D_FXSR		0x01000000
/** SSE support **/
#define CPUCAP_D_SSE		0x02000000
/** SSE2 support **/
#define CPUCAP_D_SSE2		0x04000000
/** self snoop (???) **/
#define CPUCAP_D_SS		0x08000000
/** HyperThreading technology **/
#define CPUCAP_D_HTT		0x10000000
/** thermal monitor **/
#define CPUCAP_D_TM		0x20000000
//bit 30 is reserved
/** pending break enable (???) **/
#define CPUCAP_D_PBE		0x80000000

typedef struct KiCpuInfo
{
	char mfr[13];
	struct
	{
		char stepping;
		char model;
		char family;
		char type;
		char ext_model;
		char ext_family;
		char brand;
		long c_flags;
		long d_flags;
	} features;
} KiCpuInfo;

void KiCpuId(struct KiCpuInfo *);

#endif

