/***************************************************************
 *
 * arch/i386/include/sched.h
 * Scheduling defines for AWOS
 *
 * See LICENSE file included with this source for copyright
 * information.
 *
 ***************************************************************/

/* WARNING: This code is EXTREMELY platform-dependent and internal to the
 * kernel.  If you don't need to include this file, don't.  Use more public
 * interfaces to the scheduler. */

#ifndef ___I386_SCHED_H_
#define ___I386_SCHED_H_

#include <kernel.h>
#include <unistd.h>	/* NULL */
#include <limits.h>	/* ATEXIT_MAX */
#include <cpuspec.h>	/* struct regs */

typedef void(*sched_fnptr_t)(void);

struct task
{
	/** the current memory location of the stack **/
	uint32_t *currstack;
	/** the location of the start of the stack **/
	uint32_t *stack;
	/** the process ID number of the task **/
	uint8_t pid;
	/** the image name of the task **/
	char *name;
	/** previous task in linked list **/
	struct task *prev;
	/** next task in linked list **/
	struct task *next;
	/** task's status flags **/
	char flags;
	/** command line passed to this task **/
	char *cmdline;
	/** the current errno for this task
	 ** PORTERS: errno must be of type int for POSIX **/
	int errno;
	/** the VT the task is running on, if any **/
	uint8_t console;
	/** wake-up time **/
	uint64_t wake;
	/** parent task **/
	struct task *parent;
	/** information for the exit() function **/
	struct atexitinfo
	{
		/** the array of functions exit needs to call **/
		sched_fnptr_t function[ATEXIT_MAX];
		/** the number of functions already registered with atexit() **/
		uint8_t functions_reged;
	} atexit_info;
};

typedef struct task * task_t;

/** TASK FLAGS **/
/*** The task is runnable ***/
#define TSK_RUNNABLE	0x01
/*** The task is killed ***/
#define TSK_KILLED	0x02
/*** Shh... It's sleeping... ***/
#define TSK_SLEEPING	0x04
/*** User-mode task ***/
#define TSK_USER	0x08

void		__KiUpdateTSS(struct regs *r);
uint32_t *	__KiGetNextTask(uint32_t *);
task_t		__KiCreateTask(uint32_t, char *);
task_t		__KiCreateTaskEx(uint32_t, char *, char, char *, uint32_t);

extern void 	yield(void);
extern void	exit(int);

uint64_t	get_time(void);
void		csleep(uint32_t cyc);

extern task_t currtask;
extern task_t idletask;

#endif /* !___I386_SCHED_H_ */

