/***************************************************************
 *
 * drivers/input/kbdinit.c
 * Keyboard initialisation routines for AWOS
 *
 * See LICENSE file included with this source for copyright
 * information.
 *
 ***************************************************************/

#include <portio.h>
#include <kernel.h>
#include <stdint.h>
#include <input.h>

int kbd_keybuf[256];
uint8_t kbd_keybuf_idx;
uint8_t kbd_keybuf_end;

uint32_t kbd_flags;

/*!\fn void KbdInit(void)
 * \brief Detects and initialises the keyboard (platform-independent)
 * \return Nothing.
 * \note The goal of this function is to work on every platform
 * \note i.e. x86, ppc, alpha -- so we do probing
 * \warning hopefully r/w port 60h isn't deadly on other plats?
 */
void KbdInit(void)
{
	kbd_keybuf_idx = 0;
	kbd_keybuf_end = 0;

	if(i8042_kbd_detect())
	{
		printf("kbd: found i8042\n");
		i8042_kbd_init();
		return;
	};
	
	/* TODO: more kbds */
	
	printf("kbd: could not find keyboard\n");
	
	return;
}

/*!\fn int KbdReadKey(void)
 * \brief Reads a key from the keyboard buffer; 0 = none.
 * \return Key which was pressed.
 * \note Should not be called directly, but via getch() and the like.
 */
int KbdReadKey(void)
{
	if(kbd_keybuf_idx == kbd_keybuf_end)
		return 0;
	
	return kbd_keybuf[kbd_keybuf_idx++];
}

/*!\fn void KbdPushKey(int c)
 * \brief Pushes a key to the keyboard buffer.
 * \param c Key to be pushed.
 * \return Nothing.
 */
void KbdPushKey(int c)
{
	if(c != 0)
		kbd_keybuf[kbd_keybuf_end++] = c;
}

