/***************************************************************
 *
 * arch/i386/mm/malloc.c
 * kernel memory allocator for AWOS
 *
 * See LICENSE file included with this source for copyright
 * information.
 *
 ***************************************************************/

#include <kernel.h>
#include <stddef.h>
#include <stdint.h>
#include <sys/types.h>
#include <kalloc.h>
#include <string.h>	/* memset */

void *malloc(size_t size)
{
	void *blk;
	blk_header_t *head;
	size_t true_size = size + sizeof(blk_header_t);

	for(head = blk_list->next; head != blk_list; head = head->next)
	{
		/* user block? */
		if(!(head->flags & BLK_USER)) continue;
		
		/* free block? */
		if(!(head->flags & BLK_FREE)) continue;
		
		/* will the amount asked for fit in this block? */
		if(head->size >= true_size)
		{
			/* is this _EXACTLY_ what we need? */
			if(head->size == true_size)
			{
				/* no longer free */
				head->flags &= ~BLK_FREE;
				/* remove from blk_list */
				head->prev->next = head->next;
				head->next->prev = head->prev;
				blk = (void *)head + sizeof(blk_header_t);
				return blk;
			} else {
				/* will hold the block we're chopping off */
				blk_header_t *newblk;

				/* size of "parent" block decremented */
				head->size -= true_size;
				/* newblk points to new child's location */
				newblk = ((void *)head) + head->size;
				newblk->size = true_size;
				/* flags are all clear */
				newblk->flags = head->flags & ~BLK_FREE;
				/* set linked list members */
				newblk->prev = head;
				newblk->next = head->next;
				newblk->prev->next = newblk;
				newblk->next->prev = newblk;
				
				/* child is born -- send it off to the world */
				blk = (void *)newblk + sizeof(blk_header_t);
				return blk;
			};
		};
	};

	/* oops, no block big enough.  errno should be ENOMEM. */
	printf("not enough memory for %d bytes allocation\n"\
			"blk_list at %08p: prev %08p, next %08p, flags %d, size %d\n", true_size, blk_list, blk_list->prev, blk_list->next, blk_list->flags, blk_list->size);
	return NULL;
};

void *calloc(size_t size)
{
	void *ptr;

	ptr = malloc(size);

	return memset(ptr, 0x00, size);
};

