/***************************************************************
  *
  * net/hal/hal_stack.h
  * Hardware Abstraction Layer for the network stack
  *
  * See LICENSE file included with this source for copyright
  * information.
  *
 ***************************************************************/

/* This HAL exists to make driver and network stack implementation
 * as seperate as possible. This will keep the network stack card-
 * agnostic, hopefully.
 *
 * The network drivers should NOT include this file. They should
 * instead include hal_driver.h and implement the functions listed
 * in there themselves (that is the network card specific portion).
 * This file is for the IP stack implementation only; it provides
 * basic data sending/recieving primitives that are then passed
 * along to the network card.
 */

#ifndef AWOS_HAL_STACK_H
#define AWOS_HAL_STACK_H

// Structures for IP retrevial
struct ip_list {
	char *ip;               // IP address
	struct *ip_list;        // Pointer to next node
                                // (yes, it's a linked list)
};

// Status or error codes for transmit 
#define XMIT_SUCCESS 0
#define XMIT_FAIL 1
#define XMIT_BUFFULL 2
#define XMIT_RETRY 3
#define XMIT_NOTREADY 4
#define XMIT_READY 5

// Status or error codes for protocol errors 
#define PROTO_SUCCESS 	0x0

// TCP
#define PROTO_TCP_FIN	0x01
#define PROTO_TCP_SYN	0x02
#define PROTO_TCP_RST	0x04
#define PROTO_TCP_PUSH	0x08
#define PROTO_TCP_ACK	0x10
#define PROTO_TCP_URG	0x20
#define PROTO_TCP_ECE	0x40
#define PROTO_TCP_CWR	0x80
#define PROTO_TCP_FLAGS	(PROTO_TCP_FIN|PROTO_TCP_SYN|PROTO_TCP_RST| \
			PROTO_TCP_PUSH|PROTO_TCP_ACK|PROTO_TCP_URG| \
			PROTO_TCP_ECE|PROTO_TCP_CWR)


// IP functions
int hal_assign_ip (int size, char *data);
int hal_retrieve_ip (int size, struct ip_list *data);

// MAC functions
int hal_retrieve_mac (int size, char *data);
int hal_set_mac (int size, char *data);

#endif
