/***************************************************************
 *
 * net/hal/iface-mgr.cxx
 * Interface Manager for ADM-compatible NIC drivers
 *
 * See LICENSE file included with this source for copyright
 * information.
 *
 ***************************************************************/

#include "hal_driver.h"
#include "iface-mgr.h"

static Net_IfaceDriver **interfaces;
static uint8_t nextIfaceId;

// These functions MUST be accessible by our C friends.
extern "C"
{
	#include <mutex.h>

	void NetHalInterfaceStart(void)
	{
		interfaces = (Net_IfaceDriver **)kmalloc(sizeof(Net_IfaceDriver *));	//dumb && necessary
		nextIfaceId = 0;
		return;
	};

	/*! \fn NetHalInterfaceAdd
	 * \param interface The interface to add.
	 * \notes This function expects an interface that has not yet been
	 *        initialised.
	 */
	int NetHalInterfaceAdd(Net_IfaceDriver *interface)
	{
		if(! (interface->InitializeDriver()) )
			return -1;	// ERROR. we can't add.

		while(!MxGetMutex(MUTEX_NET)) yield();
		
		nextIfaceId++;

		__NetHalRealloc();

		interfaces[nextIfaceId] = interface;

		MxReleaseMutex(MUTEX_NET);

		return nextIfaceId;
	};

	Net_IfaceDriver *NetHalGetInterface(uint32_t interface)
	{
		if(interface > nextIfaceId)
			return 0;	// Invalid if#

		return interfaces[interface];
	};

	void __NetHalRealloc(void)
	{
		interfaces = (Net_IfaceDriver **)krealloc(interfaces, sizeof(Net_IfaceDriver *) * nextIfaceId);
	};
};

