#include "ne2000.h"

extern "C"
{
#include <pci.h>
#include <kernel.h>
#include <portio.h>

#include "../inet/ip_header.h"

};

Net_NE2000IfDrv::Net_NE2000IfDrv()
{
	dprintf("NE2000 PCI network card driver version 0.0.1\n");
	
	pciDevicesSupported[0].vendor  = 0x10EC;
	pciDevicesSupported[0].product = 0x8029; // Realtek 8029

	pciDevicesSupported[1].vendor  = 0x1050;
	pciDevicesSupported[1].product = 0x0940; // Winbond 89C940F

	pciDevicesSupported[2].vendor  = 0x1050;
	pciDevicesSupported[2].product = 0x5A5A; // Winbond 89C940F 

	pciDevicesSupported[3].vendor  = 0x1106;
	pciDevicesSupported[3].product = 0x0926; // VIA Technologies VT86C926

	pciDevicesSupported[4].vendor  = 0x10BD;
	pciDevicesSupported[4].product = 0x0E34; // Surecom NE-34

	pciDevicesSupported[5].vendor  = 0x4A14;
	pciDevicesSupported[5].product = 0x5000; // NetVin 5000

	pciDevicesSupported[6].vendor  = 0x11F6;
	pciDevicesSupported[6].product = 0x1401; // Compex

	pciDevicesSupported[7].vendor  = 0x8C4A;
	pciDevicesSupported[7].product = 0x1980; // ProLAN

	pciDevicesSupported[8].vendor  = 0x8E2E;
	pciDevicesSupported[8].product = 0x3000; // KTI

	this->name = (char *) "NE2000 PCI card";
};

Net_NE2000IfDrv::~Net_NE2000IfDrv()
{
};

void Net_NE2000IfDrv::SetRegisterPage(unsigned int page)
{
	char b;
	
	// Sanity check
	if(page >= 4)
		return;
	
	b = inb(basePort + 0x00);
	dprintf("ne2000: inb 1 = %02X\n", b);
	b &= 0x3F;
	b |= (page<<6) & 0xC0;
	outb(basePort + 0x00, b);
	b = inb(basePort + 0x00);
	dprintf("ne2000: inb 2 = %02X\n", b);
};

BOOL Net_NE2000IfDrv::InitializeDriver()
{
	BOOL keepSearching = TRUE;
	int i;
	
	dprintf("ne2000: searching for card");
	pciDevice = NULL;
	
	while(keepSearching)
	{
		dprintf(".");
		pciDevice = PciFindByClassB(pciDevice, 2, 0);
		if(pciDevice == NULL)
			return FALSE;

		for(int i = 0; i < DEVS_SUPP; i++)
		{
			if(pciDevice->dev.device_id == pciDevicesSupported[i].product
			&& pciDevice->dev.vendor_id == pciDevicesSupported[i].vendor)
			{
				keepSearching = FALSE;
				break;
			};
		};
	};
	
	dprintf("found\n");
	
	// Get basePort / irq
	basePort = pciDevice->dev.bar[0] & ~0x3;
	irq = pciDevice->dev.irq_line;
	dprintf("ne2000: portbase %04X, irq %i\n", basePort, irq);
	
	// OK, most of this stuff below will need to be rewritten,
	// as this is pretty much spoonfed.
	// Reset card
	outb(basePort + 0x1F, inb(basePort + 0x1F));
	while((inb(basePort + 0x07) & 0x80) == 0)
		yield();
	dprintf("ne2000: card reset\n");
	
	// Reset interrupt flags.
	outb(basePort + 0x07, 0xFF);
	
	// Now for page 0. NODMA + STOP.
	outb(basePort + 0x00, 0x21);
	
	// Byte mode.
	outb(basePort + 0x0E, 0x21);
	
	// Clear count (low, then high).
	outb(basePort + 0x0A, 0x00);
	outb(basePort + 0x0B, 0x00);
	
	// Mask IRQs.
	outb(basePort + 0x0F, 0x00);
	outb(basePort + 0x07, 0xFF);
	
	// RX monitor.
	outb(basePort + 0x0C, 0x20);
	
	// TX loopback.
	outb(basePort + 0x0D, 0x02);
	
	// Set count to 32.
	outb(basePort + 0x0A, 0x20);
	outb(basePort + 0x0B, 0x00);
	
	// Set DMA to 0x0000.
	outb(basePort + 0x08, 0x00);
	outb(basePort + 0x09, 0x00);
	
	// OK, let's read stuff.
	outb(basePort + 0x00, 0x12);
	
	for(i=0; i<6; i++)
	{
		mac_addr[i] = inb(basePort + 0x10);
	};
	
	printf("ne2000: MAC address = %02X:%02X:%02X:%02X:%02X:%02X\n"
		,mac_addr[0]
		,mac_addr[1]
		,mac_addr[2]
		,mac_addr[3]
		,mac_addr[4]
		,mac_addr[5]
			);
	
	return TRUE;
};

BOOL Net_NE2000IfDrv::IsDriverInUse()
{
	return FALSE;
};

BOOL Net_NE2000IfDrv::DisposeDriver()
{
	if(IsDriverInUse())
		return FALSE;
	delete this;
	return TRUE;
};

int Net_NE2000IfDrv::PushData(int type, void *data)
{
	(void) type;
	(void) data;

	return 0;
};

int Net_NE2000IfDrv::RetrData(int type, void *data)
{
	(void) type;
	(void) data;

	return 0;
};

uint32_t Net_NE2000IfDrv::GetDeviceFlags(void)
{
	return 0;
};

BOOL Net_NE2000IfDrv::IsFlagSet(uint32_t flag)
{
	(void) flag;
	return FALSE;
};

BOOL Net_NE2000IfDrv::SetDeviceFlag(uint32_t flag)
{
	(void) flag;
	return FALSE;
};
