#include "../hal/hal_driver.h"

extern "C"
{
#include <pci.h>
}

/** registers **/
#define RTL8139_REG_RXBUF	0x30
#define RTL8139_REG_CMD		0x37
#define RTL8139_REG_CAPR	0x38
#define RTL8139_REG_CBR		0x3A
#define RTL8139_REG_IMR		0x3C
#define RTL8139_REG_ISR		0x3E
#define RTL8139_REG_TXCFG	0x40
#define RTL8139_REG_RXCFG	0x44

/** commands **/
#define RTL8139_CMD_TXON	0x04
#define RTL8139_CMD_RXON	0x08
#define RTL8139_CMD_RESET	0x10

/** Interrupt Status Bits **/
#define RTL8139_ISB_RXOK	0x01
#define RTL8139_ISB_RXERR	0x02
#define RTL8139_ISB_TXOK	0x04
#define RTL8139_ISB_TXERR	0x08
#define RTL8139_ISB_RXOF	0x10
#define RTL8139_ISB_RXUF	0x20
#define RTL8139_ISB_RXFIFOOF	0x40
#define RTL8139_ISB_TIMEOUT	0x4000
#define RTL8139_ISB_SYSERR	0x8000

/** TX config bits **/
#define RTL8139_TXC_MXDMA0	0x100
#define RTL8139_TXC_MXDMA1	0x200
#define RTL8139_TXC_MXDMA2	0x400
#define RTL8139_TXC_IFG0	0x1000000
#define RTL8139_TXC_IFG1	0x2000000

/** RX config bits **/
#define RTL8139_RXC_APM		0x0002
#define RTL8139_RXC_AM		0x0004
#define RTL8139_RXC_AB		0x0008
#define RTL8139_RXC_MXDMA0	0x0100
#define RTL8139_RXC_MXDMA1	0x0200
#define RTL8139_RXC_MXDMA2	0x0400
#define RTL8139_RXC_RBLEN0	0x0800
#define RTL8139_RXC_RBLEN1	0x1000

class Net_Rtl8139IfDrv : public Net_IfaceDriver
{
public:
	Net_Rtl8139IfDrv();
	~Net_Rtl8139IfDrv();

	BOOL InitializeDriver();
	BOOL IsDriverInUse();
	BOOL DisposeDriver();

	int PushData(int, void *);
	int RetrData(int, void *);

	uint32_t GetDeviceFlags();
	BOOL IsFlagSet(uint32_t);
	BOOL SetDeviceFlag(uint32_t);
private:
	//uint32_t GetDeviceStatus();
	
	struct pcidev_ll *pciDevice;
	
	static const int DEVS_SUPP = 12;
	
	struct {
		uint16_t vendor;
		uint16_t product;
	} pciDevicesSupported[DEVS_SUPP];
	
	void SetRegisterPage(unsigned int);
	
	//uint32_t *mem_start;
	void *rxBuffer;
};

